#include <stdio.h>
#include "xdb_searcher.h"
#include "ip2region.h"

char *ip2region(char *xdb_file, char *ip)
{
    char *db_path = xdb_file;
    xdb_vector_index_t *v_index;
    xdb_searcher_t searcher;
    char region_buffer[256];
    //long s_time;

    // 1、从 db_path 加载 VectorIndex 索引。
    // 得到 v_index 做成全局缓存，便于后续反复使用。
    // 注意：v_index 不需要每次都加载，建议在服务启动的时候加载一次，然后做成全局资源。
    v_index = xdb_load_vector_index_from_file(db_path);
    if (v_index == NULL) {
        printf("failed to load vector index from `%s`\n", db_path);
        return NULL;
    }
    
    // 2、使用全局的 VectorIndex 变量创建带 VectorIndex 缓存的 xdb 查询对象
    int err = xdb_new_with_vector_index(&searcher, db_path, v_index);
    if (err != 0) {
        printf("failed to create vector index cached searcher with errcode=%d\n", err);
        return NULL;
    }
    
    // 3、调用 search API 查询
    // 得到的 region 信息会存储到 region_buffer 里面，如果你自定义了数据，请确保给足 buffer 的空间。
    //s_time = xdb_now();
    err = xdb_search_by_string(&searcher, ip, region_buffer, sizeof(region_buffer));
    if (err != 0) {
        printf("failed search(%s) with errno=%d\n", ip, err);
        return NULL;
    } else {
        ;
        //printf("{region: %s, took: %dμs}", region_buffer, (int)(xdb_now() - s_time));
    }

    // 备注：并发使用，没一个线程需要单独定义并且初始化一个 searcher 查询对象。

    // 4、关闭 xdb 查询器，如果是要关闭服务，也需要释放 v_index 的内存。
    xdb_close(&searcher);
    xdb_close_vector_index(v_index);
    
    return strdup(region_buffer);
}
